// **********************************************************************
//
// Copyright (c) 2001 MT Tools.
//
// All Rights Reserved
//
// MT_DORB is based in part on the product DORB,
// written by Shadrin Victor
//
// See Readme.txt for contact information
//
// **********************************************************************
unit context;

interface

uses
  orbtypes,env_int,Classes,code_int;

type

  TContext = class(TInterfacedObject,IContext)
  private
    FName: string;
    FParent : IContext;
    FChildren : IInterfaceList;
    FProperties : TStringList;
  protected
    procedure set_one_value(const prop_name: Identifier; const value: string);
    procedure set_values(const values : NVList);
    procedure get_values(const start_scope: Identifier; const op_flags: Flags; const prop_name: Identifier; out values : NVList);
    procedure delete_values(const prop_name: Identifier);
    procedure create_child(const ctx_name: Identifier; out child_ctx: IContext);
    procedure delete(const del_flags: Flags);
    // DORB extension
    procedure encode(const enc: IEncoder; const ctxlist: IInterfaceList);
    function decode(const dec: IDecoder): Boolean;
    function context_name(): PChar;
    function parent(): IContext;
  public
    constructor Create(const name: string; const parent: IContext = nil); overload;
    constructor Create(const dec: IDecoder); overload;
    destructor Destroy(); override;
  end;

implementation

uses
  throw,orb,exceptions;

//*********************************************************************
//  TContext
//*********************************************************************
constructor TContext.Create(const name: string; const parent: IContext);
begin
  inherited Create();
  FName := name;
  FParent := parent;
  FProperties := nil;
end;

constructor TContext.Create(const dec: IDecoder);
begin
  decode(dec);
end;

destructor TContext.Destroy;
begin
  FProperties.Free;
  inherited;
end;

function TContext.context_name: PChar;
begin
  result := PChar(FName)
end;

procedure TContext.create_child(const ctx_name: Identifier;
  out child_ctx: IContext);
begin
  if FChildren = nil then FChildren := TInterfaceList.Create;
  child_ctx := TContext.Create(ctx_name,Self);
  FChildren.Add(child_ctx);
end;

function TContext.decode(const dec: IDecoder): Boolean;
begin
  result := false;
  assert(false,'not implemented');
end;

procedure TContext.delete(const del_flags: Flags);
var
  i : integer;
begin
  if ((del_flags and CTX_DELETE_DESCENDENTS) = 0) and Assigned(FChildren) and (FChildren.Count > 0) then
    dorb_throw(SystemException.Create_type(st_BAD_PARAM));
  FProperties.Clear;
  if ((del_flags and CTX_DELETE_DESCENDENTS) <> 0) and Assigned(FChildren) then
    for i := 0 to Pred(FChildren.Count) do
      (FChildren[i] as IContext).delete(del_flags);
end;

procedure TContext.delete_values(const prop_name: Identifier);
var
  i : integer;
begin
  if (FProperties = nil) or (FProperties.Count = 0) then
    dorb_throw(SystemException.Create_type(st_BAD_CONTEXT));
  if prop_name = '*' then
    FProperties.Clear
  else
    begin
      i := FProperties.IndexOfName(prop_name);
      if i = -1 then
        dorb_throw(SystemException.Create_type(st_BAD_CONTEXT));
      FProperties.Delete(i);
    end;
end;

procedure TContext.encode(const enc: IEncoder;
  const ctxlist: IInterfaceList);
begin
  assert(false,'not implemented');
end;

procedure TContext.get_values(const start_scope: Identifier;
  const op_flags: Flags; const prop_name: Identifier; out values: NVList);
begin
  assert(false,'not implemented');
end;

function TContext.parent: IContext;
begin
  result := FParent
end;

procedure TContext.set_one_value(const prop_name: Identifier;
  const value: string);
begin
  if (prop_name = '') or (value = '') then
    dorb_throw(SystemException.Create_type(st_BAD_PARAM));
  if FProperties = nil then FProperties := TStringList.Create;
  FProperties.Values[prop_name] := value;
end;

procedure TContext.set_values(const values: NVList);
var
  i : integer;
  name,value: string;
begin
  for i := 0 to Pred(Length(values)) do
    begin
      name := values[i].name;
      if not values[i].argument.to_string(value) then
        dorb_throw(SystemException.Create_type(st_BAD_PARAM));
      set_one_value(name,value);
    end;
end;

end.
